package com.diagbot.facade;

import com.diagbot.client.AuthServiceClient;
import com.diagbot.dto.LoginDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.JWT;
import com.diagbot.entity.User;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.exception.ServiceErrorCode;
import com.diagbot.service.impl.UserServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ImgVerVerVO;
import com.diagbot.vo.UserSaveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Date;

/**
 * @Description: 用户业务层
 * @author: gaodm
 * @time: 2018/8/6 9:00
 */
@Component
public class UserFacade extends UserServiceImpl {

    @Autowired
    AuthServiceClient authServiceClient;
    @Autowired
    VerFacade verFacade;

    public User createUser(UserSaveVO userSaveVO){
        User user = new User();
        BeanUtil.copyProperties(userSaveVO, user);
        ImgVerVerVO imgVerVerVO = new ImgVerVerVO();
        BeanUtil.copyProperties(userSaveVO, imgVerVerVO);
        User bean = findByName(user.getUsername());
        if(bean != null) {
            throw new CommonException(ServiceErrorCode.USER_EXIST);
        }
        if(StringUtil.isEmpty(user.getPassword())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入密码!");
        }
        if(StringUtil.isEmpty(user.getEmail())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入邮箱!");
        }
        verFacade.verifyImgVerification(imgVerVerVO); //保存时再次校验图形验证码，主要是为了防止跳过前端校验直接调用接口

        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        this.save(user);
        return  user;
    }

    @Cacheable(value = "UserName", key = "'username:'+#p0")
    public User getUserInfo(String username){
        return this.findByName(username);
    }

    public RespDTO login(String username , String password){
        User user= this.findByName(username);
        if(null==user){
            throw new CommonException(ServiceErrorCode.USER_NOT_FOUND);
        }
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        if(!passwordEncoder.matches(password,user.getPassword())){
            throw new CommonException(ServiceErrorCode.USER_PASSWORD_ERROR);
        }

        JWT jwt = authServiceClient.getToken("Basic dWFhLXNlcnZpY2U6MTIzNDU2", "password", username, password);
        // 获得用户菜单
        if(null==jwt){
            throw new CommonException(ServiceErrorCode.GET_TOKEN_FAIL);
        }
        LoginDTO loginDTO=new LoginDTO();
        loginDTO.setUser(user);
        loginDTO.setToken(jwt.getAccess_token());
        return RespDTO.onSuc(loginDTO);
    }

    private static final String CACHE_NAME = "UserInfo";

    //缓存演示
    @CachePut(value = CACHE_NAME, key = "'user:'+#p0['id']")
    public User saveUser(User user){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        user.setGmtCreate(new Date());
        user.setCreator(UserUtils.getCurrentPrincipleID());
        super.save(user);
        return user;
    }

    @CachePut(value = CACHE_NAME, key = "'user:'+#p0['id']")
    public User updateUser(User user){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        user.setGmtModified(new Date());
        user.setModifier(UserUtils.getCurrentPrincipleID());
        super.updateById(user);
        return user;
    }

    @Override
    @Cacheable(value = CACHE_NAME, key = "'user:'+#p0")
    public User getById(Serializable id){
        return super.getById(id);
    }

    @Override
    @CacheEvict(value = CACHE_NAME, key = "'user:'+#p0")
    public boolean removeById(Serializable id){
        return super.removeById(id);
    }


}
