package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.VerFacade;
import com.diagbot.vo.ImgVerVerVO;
import com.diagbot.vo.SmsVerCreatVO;
import com.diagbot.vo.SmsVerVerVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description: 用户验证API
 * @author: gaodm
 * @time: 2018/9/4 19:30
 */
@RestController
@RequestMapping("/userver")
public class VerController {
    @Autowired
    private VerFacade verFacade;

    /**
     * 获取图片验证码
     * @return 图片验证码包括图片验证码唯一标志和图片信息
     */
    @ApiOperation(value = "获取图片验证码", notes = "获取图片验证码包括图片验证码唯一标志和图片信息")
    @GetMapping("/getImgVerification")
    @SysLogger("getImgVerification")
    public RespDTO getImgVerification(){
        return   RespDTO.onSuc(verFacade.getImgVerification());
    }

    /**
     * 验证图片验证码信息
     * @param imgVerVerVO 验证图片验证码所需参数
     * @return 是否验证成功
     */
    @ApiOperation(value = "验证图片验证码", notes = "图片验证码唯一标志后和验证码不能为空")
    @PostMapping("/verifyImgVerification")
    @SysLogger("verifyImgVerification")
    public RespDTO verImgVerification(@RequestBody @Valid ImgVerVerVO imgVerVerVO){
        return   RespDTO.onSuc(verFacade.verifyImgVerification(imgVerVerVO));
    }

    /**
     * 获取短信验证码
     * @param smsVerCreatVO 获取短信验证码输入参数
     * @return 获取短信验证码是否成功
     */
    @ApiOperation(value = "获取短信验证码", notes = "图片验证码唯一标志后，图形验证码，用户号码不能为空")
    @PostMapping("/getSmsVerification")
    @SysLogger("getSmsVerification")
    public RespDTO getSmsVerification(@RequestBody @Valid SmsVerCreatVO smsVerCreatVO){
        return   RespDTO.onSuc(verFacade.getSmsVerification(smsVerCreatVO));
    }

    /**
     * 验证短信码并且注册为新的用户
     * @param smsVerVerVO 短信验证码验证输入信息
     * @return 用户信息是否注册到数据库
     */
    @ApiOperation(value = "验证短信验证码", notes = "图片验证码唯一标志后，图形验证码，用户号码，用户验证码不能为空")
    @PostMapping("/verifySmsVerification")
    @SysLogger("verifySmsVerification")
    public RespDTO verifySmsVerification(@RequestBody SmsVerVerVO smsVerVerVO){
        return RespDTO.onSuc(verFacade.verifySmsVerification(smsVerVerVO));
    }
}
